@@ -96,9 +96,8 @@ class SingleFileStorage(Storage):
 
         for item in split_collection(text):
             item = Item(item)
-            href = self._get_href(item)
             etag = item.hash
-            self._items[href] = item, etag
+            self._items[item.ident] = item, etag
 
         return ((href, etag) for href, (item, etag) in iteritems(self._items))
 
@@ -112,7 +111,7 @@ class SingleFileStorage(Storage):
             raise exceptions.NotFoundError(href)
 
     def upload(self, item):
-        href = self._get_href(item)
+        href = item.ident
         self.list()
         if href in self._items:
             raise exceptions.AlreadyExistingError(href)
